@@ -25,7 +25,7 @@ from openpyxl.workbook import Workbook
 from openpyxl.worksheet.dimensions import ColumnDimension, DimensionHolder
 
 from .canvas_robot_model import AC_YEAR, NEXT_YEAR, ENROLLMENT_TYPES, STUDADMIN, \
-    EDUCATIONS, COMMUNITIES, LocalDAL, SHORTNAMES, CanvasConfig
+    EDUCATIONS, COMMUNITIES, LocalDAL, SHORTNAMES, CanvasConfig, EXAMINATION_FOLDER
 from .entities import User, EnrollDTO, SearchTextInCourseDTO, QuestionDTO
 
 logging.getLogger("canvasapi").setLevel(logging.WARNING)  # we don't need the info messages
@@ -106,21 +106,27 @@ class DibsaRetrieveError(Error):
 
 
 @define
-class Grade(object):
+class Grade:
     stud_name: str
     stud_id: str
     final_score: float
     final_grade: float
 
+@define
+class ExaminationDTO:
+    course_id: int
+    course_name: str
+    name:str
+
 
 @define
-class LabelType(object):
+class LabelType:
     label: str
     field_type: str
 
 
 @define
-class Profile(object):
+class Profile:
     login_id: str
     id: int
     name: str
@@ -177,6 +183,9 @@ class CourseMetadata:
     nr_assignments: int
     nr_quizzes: int
     nr_files: int
+    assignments_summary: str
+    examinations_summary: str
+    examination_candidates: str
     # nr_collaborations: int
 
     # #ext_urls: int
@@ -187,8 +196,9 @@ class CourseMetadata:
 class CanvasRobot(object):
     db: callable
     TOT_WEIGHT: int = 100
-
-    def __init__(self, reset_api_keys=False):
+    year: int = AC_YEAR
+    def __init__(self, reset_api_keys=False, years_back = 0):
+        self.year = AC_YEAR - years_back
         config = CanvasConfig(reset_api_keys=reset_api_keys)
         self.canvas = canvasapi.Canvas(config.url, config.api_key)
         self.admin = self.canvas.get_account(config.admin_id) if config.admin_id else None
@@ -221,7 +231,7 @@ class CanvasRobot(object):
         :returns canvas courses for current user in role"""
         return self.canvas.get_courses(enrollment_type=enrollment_type)
 
-    def get_courses_in_account(self, by_teachers: list, this_year=True):
+    def get_courses_in_account(self, by_teachers: list=None, this_year=True):
         """
         get all course in account here use_is has the role/type [enrollment_type]
         :param by_teachers: list of teacher id's
@@ -233,7 +243,7 @@ class CanvasRobot(object):
         for course in self.admin.get_courses(by_teachers=by_teachers):
             # only show/insert/update course if current year
 
-            if this_year and (str(course.sis_course_id)[:4] != str(AC_YEAR)
+            if this_year and (str(course.sis_course_id)[:4] != str(self.year)
                               or course.name.endswith('conclude')):
                 continue
             courses.append(course)
@@ -296,12 +306,12 @@ class CanvasRobot(object):
             # print("There are {} modules in page".format(len(page)))
         return
 
-    def course_metada(self, course_id):
+    def course_metadata(self, course_id, examination_names=None):
         """
         return dict with metadata of this course
         as a dict.
         :returns md: CourseMetadata"""
-
+        examination_names = examination_names or []
         course = self.get_course(course_id)
         modules = course.get_modules()
 
@@ -322,11 +332,44 @@ class CanvasRobot(object):
         pages = filter(lambda p: p.title[0:3] != 'UVT', pages)
         nr_pages = len(list(pages))
         assignments = course.get_assignments()
+        assignments_summary = "Assignments (online_uploads have details)\n"
+        examination_candidates = []
+        for assignment in assignments:
+            submissions_summary = ""
+            submissions = assignment.get_submissions()
+            for submission in submissions:
+                if submission.submission_type == "online_upload":
+                    if assignment.name in examination_names:
+                        submissions_summary += "Examination:"
+                    else:
+                        submissions_summary+="Possible Examination:"
+
+                        candidate = ExaminationDTO(course_id,
+                                                   course.name,
+                                                   assignment.name)
+
+                        examination_candidates.append(candidate)
+                    submissions_summary+=(f"/ngraded "
+                                          f"{submission.grade} at "
+                                          f"{submission.graded_at}")
+            assignments_summary+=assignment.name + ":"
+            assignments_summary+=submissions_summary
         nr_assignments = len(list(assignments))
         quizzes = course.get_quizzes()
         nr_quizzes = len(list(quizzes))
         files = course.get_files()
         nr_files = len(list(files))
+        examinations_summary = ""
+        examination_files = 0
+        for file in files:
+            folder = course.get_folder(file.folder_id)
+            if f"/{EXAMINATION_FOLDER}" in folder.full_name:
+                examination_files+=1
+                examinations_summary+=f"\n{file.display_name}"
+        if examination_files == 0:
+            examinations_summary += f"\nNo examination_files in {EXAMINATION_FOLDER}"
+        else:
+            examinations_summary += f"\nTotal: {examination_files}"
         # was this working earlier 2.2.0 ?
         # try:
         #    collaborations = course.get_collaborations()
@@ -340,8 +383,11 @@ class CanvasRobot(object):
                             nr_assignments,
                             nr_quizzes,
                             nr_files,
-                            # nr_collaborations
+                            assignments_summary,
+                            examinations_summary,
+                            examination_candidates
                             )
+        # examination_candidates: collect to create a canonical list
         return md
 
     def enroll_in_course(self,
@@ -378,7 +424,7 @@ class CanvasRobot(object):
 
         def cur_year_active(course):
             """ filter function"""
-            return (str(course.sis_course_id)[:4] == str(AC_YEAR) and
+            return (str(course.sis_course_id)[:4] == str(self.year) and
                     not course.name.endswith('conclude'))
 
         def set_id_to_course_id(course):
@@ -388,7 +434,7 @@ class CanvasRobot(object):
         if from_db:
             db = self.db
             courses = db(
-                (db.course.ac_year == AC_YEAR)).select(db.course.ALL)  # field id is db id
+                (db.course.ac_year == self.year)).select(db.course.ALL)  # field id is db id
             # map(set_id_to_course_id, courses)
         else:
             courses = self.tst.get_courses()
@@ -403,7 +449,7 @@ class CanvasRobot(object):
         osiris_id in current ac. year"""
         for course in self.tst.get_courses():
             # only consider course if current year
-            if str(course.sis_course_id)[:4] != str(AC_YEAR):
+            if str(course.sis_course_id)[:4] != str(self.year):
                 continue
             # if not hasattr(course, "course_code") or course.course_code is None:
             #     continue
@@ -413,7 +459,7 @@ class CanvasRobot(object):
     def get_course_id_using_osiris_id_from_db(self, osiris_id: str):
         db = self.db
         rows = db((db.course.course_code == osiris_id) &
-                  (db.course.ac_year == AC_YEAR)).select(db.course.course_id)
+                  (db.course.ac_year == self.year)).select(db.course.course_id)
         if rows:
             return rows[0].course_id
         return 0
@@ -522,7 +568,7 @@ class CanvasRobot(object):
         # idea: filter course of an education using db
         for course in self.tst.get_courses():
             # only insert/update course if current year
-            if str(course.sis_course_id)[:4] != str(AC_YEAR):
+            if str(course.sis_course_id)[:4] != str(self.year):
                 continue
             observers = course.get_users(enrollment_type="observer")
             for observer in observers:
@@ -540,7 +586,7 @@ class CanvasRobot(object):
 
         for course in self.tst.get_courses():
             # only get a course if current year
-            if str(course.sis_course_id)[:4] != str(AC_YEAR):
+            if str(course.sis_course_id)[:4] != str(self.year):
                 continue
             enrollments = course.get_enrollments(type="ObserverEnrollment")
             # specifying user_id is not allowed
@@ -714,7 +760,7 @@ class CanvasRobot(object):
         :return: features-matrix, labels """
 
         db = self.db  # cosmetic reasons
-        qry = db.course.ac_year == AC_YEAR
+        qry = db.course.ac_year == self.year
         # couluns = ()
         courses = db(qry).select(db.course.course_id,
                                  db.course.nr_modules,
@@ -736,7 +782,7 @@ class CanvasRobot(object):
         :return: rows/list of dicts """
 
         db = self.db  # cosmetic reasons
-        suffix = '-{}-'.format(AC_YEAR)
+        suffix = '-{}-'.format(self.year)
         qry = db.bbcourse.course_suffix.contains(suffix)
         if single_course:
             qry &= db.bbcourse.course_base == single_course
@@ -839,7 +885,7 @@ class CanvasRobot(object):
         """ get documents/attachments from db als a list
         """
         db = self.db
-        suffix = '-{}-'.format(AC_YEAR)
+        suffix = '-{}-'.format(self.year)
         try:
             items = db((db.bbcourse.course_suffix.contains(suffix)) &
                        (db.bbdocument.bbcourse_id == db.bbcourse.id)).select(db.bbdocument.ALL)
@@ -887,23 +933,60 @@ class CanvasRobot(object):
         db.commit()
         return counters
 
+    def get_examinations_from_database(self, candidate=False):
+        db = self.db
+        records = db((db.examination.id>0)&
+                     (db.examination.candidate==candidate)
+                     ).select(db.examination.ALL)
+        return records
+    def get_courses_from_database(self):
+        db = self.db
+        records = db(db.course.id>0).select(db.course.ALL)
+        return records
+    def get_course_from_database(self,course_id):
+        db = self.db
+        record = db(db.course.course_id==course_id).select(db.course.ALL).first()
+        return record
+    def delete_course_from_database(self,course_id):
+        db = self.db
+        result = db(db.course.course_id==course_id).delete()
+        db.commit()
+        return result
+
+    def update_record_db(self, search_field, search_id, table ,field, value):
+        db = self.db
+        ud_fields = {field: value}
+        row = db(db[table][search_field] == search_id).select()
+        row2 = db(db.course.course_id == search_id).select()
+        result = db(db[table][search_field] == search_id).update(**ud_fields)
+        db.commit()
+
     # noinspection PyUnusedLocal
-    def update_database_from_canvas(self):
+    def update_database_from_canvas(self, single_course=None):
         """
             Using the canvasapi to read the list of TST courses and
-            - put internal_id course_id, fname and instructors in the
+            - record internal_id course_id, fname and instructors in the
             table course
             - put teacher details in table user
+            - collects info about assignments in assignment_summary
+            - collects info about examinations in examination_summary
+            - reports new tentamination candidates
             :return number of added/updated rows
             """
 
         db = self.db
-        logging.info('open courselist')
+        examinations = self.get_examinations_from_database(candidate=False)
+
+
+        logging.info(f'open courselist for year {self.year} - {self.year+1}')
         num_rows = 0
         # tst = self.canvas.get_account(6)  # admin account
-        for course in self.admin.get_courses():
-            # only insert/update course if current year
-            if str(course.sis_course_id)[:4] != str(AC_YEAR) or course.name.endswith('conclude'):
+        courses=[self.get_course(single_course),] if single_course else self.admin.get_courses()
+
+        for course in courses:
+            # only insert/update course if current year unless single_course
+            if (str(course.sis_course_id)[:4] != str(self.year)
+                    or course.name.endswith('conclude')) and not single_course:
                 continue
             logging.info("course: {}".format(course.name))  # course
             teachers = course.get_users(enrollment_type="teacher")
@@ -941,7 +1024,8 @@ class CanvasRobot(object):
             format_str = "%Y-%m-%dT%H:%M:%SZ"
             creation_date = datetime.strptime(course.created_at, format_str)
             course_id = None
-            md = self.course_metada(course.id)
+            canonical_examination_names = [row.name for row in examinations if row.course==course_id]
+            md = self.course_metadata(course.id, canonical_examination_names)
             try:
                 course_id = db.course.update_or_insert(db.course.course_id == course.id,
                                                        course_id=course.id,
@@ -951,12 +1035,14 @@ class CanvasRobot(object):
                                                        # year course_code and suffixes
                                                        name=course.name,
                                                        creation_date=creation_date,
-                                                       ac_year=AC_YEAR,
+                                                       ac_year=self.year,
                                                        nr_modules=md.nr_modules,
                                                        nr_module_items=md.nr_module_items,
                                                        nr_pages=md.nr_pages,
                                                        nr_assignments=md.nr_assignments,
                                                        nr_quizzes=md.nr_quizzes,
+                                                       assignments_summary= md.assignments_summary,
+                                                       examinations_summary = md.examinations_summary,
                                                        teachers=teacher_logins,
                                                        teachers_names=teacher_names)
             except Exception as e:
@@ -966,7 +1052,12 @@ class CanvasRobot(object):
                 db(db.course.id == inserted_id).update(status=2)
             else:
                 course_id = db(db.course.course_id == course.id).select().first().id
-
+            for cand in md.examination_candidates:
+                db.examination.update_or_insert((db.examination.course == cand.course_id)&
+                                                 (db.examination.name == cand.name),
+                                                 course=cand.course_id,
+                                                 course_name=cand.course_name,
+                                                 name=cand.name)
             for user_id in teachers_ids:
                 db.course2user.update_or_insert((db.course2user.course == course_id) &
                                                 (db.course2user.user == user_id),
@@ -1219,7 +1310,8 @@ class CanvasRobot(object):
                             LabelType(label='score', field_type='percentage'),
                             LabelType(label='grade', field_type='grade')]
         styles = {
-            # not complete see https://www.web2pyref.com/reference/field-type-database-field-types
+            # list below is not complete
+            # see https://www.web2pyref.com/reference/field-type-database-field-types
             'datetime': NamedStyle(name='date', number_format='yyyy-mm-dd'),
             'id': NamedStyle(name='int', number_format='#,##0'),
             'integer': NamedStyle(name='int', number_format='#,##0'),
